"""Contains RTK models for robot location and RTK positions."""

from dataclasses import dataclass


@dataclass
class Point:
    """Returns a lat long."""

    latitude: float
    longitude: float

    def __init__(self, latitude=0.0, longitude=0.0):
        self.latitude = latitude
        self.longitude = longitude


@dataclass
class Dock(Point):
    """Stores robot dock position."""

    rotation: int

    def __init__(self):
        super().__init__()
        self.rotation = 0


@dataclass
class Location:
    """Stores/retrieves RTK GPS data."""

    device: Point
    RTK: Point
    dock: Dock
    position_type: int
    orientation: int # 360 degree rotation +-

    def __init__(self):
        self.device = Point()
        self.RTK = Point()
        self.dock = Dock()



from dataclasses import dataclass, field

@dataclass
class ConnectData:
    connect_type: int = 0
    ble_rssi: int = 0
    wifi_rssi: int = 0

    @classmethod
    def from_dict(cls, data: dict):
        return cls(
            connect_type=data.get('connect_type', 0),
            ble_rssi=data.get('ble_rssi', 0),
            wifi_rssi=data.get('wifi_rssi', 0)
        )

@dataclass
class DeviceData:
    sys_status: int = 0
    charge_state: int = 0
    battery_val: int = 0
    sensor_status: int = 0
    last_status: int = 0
    sys_time_stamp: str = ""

    @classmethod
    def from_dict(cls, data: dict):
        return cls(
            sys_status=data.get('sys_status', 0),
            charge_state=data.get('charge_state', 0),
            battery_val=data.get('battery_val', 0),
            sensor_status=data.get('sensor_status', 0),
            last_status=data.get('last_status', 0),
            sys_time_stamp=data.get('sys_time_stamp', "")
        )

@dataclass
class RTKData:
    status: int = 0
    pos_level: int = 0
    gps_stars: int = 0
    dis_status: str = ""
    co_view_stars: int = 0

    @classmethod
    def from_dict(cls, data: dict):
        return cls(
            status=data.get('status', 0),
            pos_level=data.get('pos_level', 0),
            gps_stars=data.get('gps_stars', 0),
            dis_status=data.get('dis_status', ""),
            co_view_stars=data.get('co_view_stars', 0)
        )

@dataclass
class LocationData:
    real_pos_x: int = 0
    real_pos_y: int = 0
    real_toward: int = 0
    pos_type: int = 0
    bol_hash: str = ""

    @classmethod
    def from_dict(cls, data: dict):
        return cls(
            real_pos_x=data.get('real_pos_x', 0),
            real_pos_y=data.get('real_pos_y', 0),
            real_toward=data.get('real_toward', 0),
            pos_type=data.get('pos_type', 0),
            bol_hash=data.get('bol_hash', "")
        )

@dataclass
class WorkData:
    path_hash: str = ""
    progress: int = 0
    area: int = 0
    bp_info: int = 0
    bp_hash: str = ""
    bp_pos_x: int = 0
    bp_pos_y: int = 0
    real_path_num: str = ""
    ub_zone_hash: str = ""
    init_cfg_hash: str = ""
    ub_ecode_hash: str = ""
    knife_height: int = 0

    @classmethod
    def from_dict(cls, data: dict):
        return cls(
            path_hash=data.get('path_hash', ""),
            progress=data.get('progress', 0),
            area=data.get('area', 0),
            bp_info=data.get('bp_info', 0),
            bp_hash=data.get('bp_hash', ""),
            bp_pos_x=data.get('bp_pos_x', 0),
            bp_pos_y=data.get('bp_pos_y', 0),
            real_path_num=data.get('real_path_num', ""),
            ub_zone_hash=data.get('ub_zone_hash', ""),
            init_cfg_hash=data.get('init_cfg_hash', ""),
            ub_ecode_hash=data.get('ub_ecode_hash', ""),
            knife_height=data.get('knife_height', 0)
        )

@dataclass
class ReportData:
    connect: ConnectData = field(default_factory=ConnectData)
    dev: DeviceData = field(default_factory=DeviceData)
    rtk: RTKData = field(default_factory=RTKData)
    locations: list[LocationData] = field(default_factory=list)
    work: WorkData = field(default_factory=WorkData)

    @classmethod
    def from_dict(cls, data: dict):
        return cls(
            connect=ConnectData.from_dict(data.get('connect', {})),
            dev=DeviceData.from_dict(data.get('dev', {})),
            rtk=RTKData.from_dict(data.get('rtk', {})),
            locations=[LocationData.from_dict(loc) for loc in data.get('locations', [])],
            work=WorkData.from_dict(data.get('work', {}))
        )


"""Manage state from notifications into MowingDevice."""

import betterproto

from pymammotion.data.model.device import MowingDevice
from pymammotion.event.event import DataEvent
from pymammotion.proto.luba_msg import LubaMsg
from pymammotion.proto.mctrl_nav import NavGetCommDataAck


class StateManager:
    """Manage state."""

    _device: MowingDevice
    gethash_ack_callback: DataEvent
    get_commondata_ack_callback: DataEvent

    def __init__(self, device: MowingDevice):
        self._device = device
        self.gethash_ack_callback = DataEvent()
        self.get_commondata_ack_callback = DataEvent()

    def get_device(self) -> MowingDevice:
        """Get device."""
        return self._device

    def set_device(self, device: MowingDevice):
        """Set device."""
        self._device = device

    async def notification(self, message: LubaMsg):
        """Handle protobuf notifications."""
        res = betterproto.which_one_of(message, "LubaSubMsg")

        match res[0]:
            case "nav":
                await self._update_nav_data(message)
            case "sys":
                self._update_sys_data(message)
            case "driver":
                self._update_driver_data(message)
            case "net":
                self._update_net_data(message)
            case "mul":
                self._update_mul_data(message)
            case "ota":
                self._update_ota_data(message)

    async def _update_nav_data(self, message):
        """Update nav data."""
        nav_msg = betterproto.which_one_of(message.nav, "SubNavMsg")
        match nav_msg[0]:
            case "toapp_gethash_ack":
                self._device.map.obstacle = dict()
                self._device.map.area = dict()
                self._device.map.path = dict()
                await self.gethash_ack_callback.data_event(nav_msg[1])
            case "toapp_get_commondata_ack":
                common_data: NavGetCommDataAck = nav_msg[1]
                updated = self._device.map.update(common_data)
                if updated:
                    await self.get_commondata_ack_callback.data_event(common_data)

    def _update_sys_data(self, message):
        """Update system."""
        sys_msg = betterproto.which_one_of(message.sys, "SubSysMsg")
        match sys_msg[0]:
            case "system_update_buf":
                self._device.buffer(sys_msg[1])
            case "toapp_report_data":
                self._device.update_report_data(sys_msg[1])
            case "mow_to_app_info":
                self._device.mow_info(sys_msg[1])

    def _update_driver_data(self, message):
        pass

    def _update_net_data(self, message):
        pass

    def _update_mul_data(self, message):
        pass

    def _update_ota_data(self, message):
        pass
